import re
import requests
from telegram import Update
from telegram.ext import Application, MessageHandler, CommandHandler, ChatMemberHandler, filters

# Bot configuration
BOT_TOKEN = '8196812859:AAH3BhJxgJdl2PMP4fAEVGnXCd8AHAx5DVk'  # Your bot token
GROUP_ID = -1002134751475  # Source group ID
BIGBUYS_TOPIC_ID = 2048976  # Topic ID for big buys

# File to store subscribed chat IDs
SUBSCRIBED_CHATS_FILE = 'bigbuys_subscribed_chats.txt'

# Always forward to this channel (even if not subscribed)
ALWAYS_FORWARD_CHANNEL = -1002414831046

# Functions to manage chat IDs in file
def load_subscribed_chats():
    """Load chat IDs from file into a set."""
    try:
        with open(SUBSCRIBED_CHATS_FILE, 'r') as f:
            return set(int(line.strip()) for line in f if line.strip())
    except FileNotFoundError:
        return set()

def save_subscribed_chats(chat_ids):
    """Save chat IDs from set to file."""
    with open(SUBSCRIBED_CHATS_FILE, 'w') as f:
        for chat_id in chat_ids:
            f.write(f"{chat_id}\n")

# Initialize set of chat IDs where bot will post
TARGET_CHAT_IDS = load_subscribed_chats()

# Command to subscribe a chat
async def start(update: Update, context):
    """Add chat to subscribers when /start is called, unless it's the source group."""
    chat_id = update.message.chat_id
    if chat_id == GROUP_ID:
        await context.bot.send_message(chat_id=chat_id, text="This is the source group. Bot cannot subscribe here.")
        return
    if chat_id not in TARGET_CHAT_IDS:
        TARGET_CHAT_IDS.add(chat_id)
        save_subscribed_chats(TARGET_CHAT_IDS)
        await context.bot.send_message(chat_id=chat_id, text="Bot will now post updates here.")
    else:
        await context.bot.send_message(chat_id=chat_id, text="Bot is already subscribed to this chat.")

# Command to unsubscribe a chat
async def stop(update: Update, context):
    """Remove chat from subscribers when /stop is called."""
    chat_id = update.message.chat_id
    if chat_id in TARGET_CHAT_IDS:
        TARGET_CHAT_IDS.remove(chat_id)
        save_subscribed_chats(TARGET_CHAT_IDS)
        await context.bot.send_message(chat_id=chat_id, text="Bot will no longer post updates here.")
    else:
        await context.bot.send_message(chat_id=chat_id, text="Bot is not subscribed to this chat.")

# Handle bot status changes (e.g., removal from group)
async def chat_member_update(update: Update, context):
    """Remove chat ID from subscribers if bot is kicked or leaves."""
    my_chat_member = update.my_chat_member
    if my_chat_member:
        new_status = my_chat_member.new_chat_member.status
        chat_id = my_chat_member.chat.id
        if new_status in ['kicked', 'left']:
            if chat_id in TARGET_CHAT_IDS:
                TARGET_CHAT_IDS.remove(chat_id)
                save_subscribed_chats(TARGET_CHAT_IDS)
                print(f"[DEBUG] Removed chat ID {chat_id} from subscribed chats.")

# Handle messages and group migrations
async def handle_message(update: Update, context):
    """Process messages from the source group and handle group migrations."""
    message = update.message
    if not message:  # Ensure message exists
        print("[DEBUG] Update received but no message found.")
        return

    # Handle group migration to supergroup
    if message.migrate_to_chat_id:
        old_chat_id = message.chat_id
        new_chat_id = message.migrate_to_chat_id
        if old_chat_id in TARGET_CHAT_IDS:
            TARGET_CHAT_IDS.remove(old_chat_id)
            TARGET_CHAT_IDS.add(new_chat_id)
            save_subscribed_chats(TARGET_CHAT_IDS)
            print(f"[DEBUG] Updated chat ID from {old_chat_id} to {new_chat_id}.")
        return

    # Check if message is from the source group and topic
    if message.chat_id != GROUP_ID:
        print(f"[DEBUG] Ignoring message from chat ID {message.chat_id} (not source group).")
        return
    if not hasattr(message, 'message_thread_id') or message.message_thread_id != BIGBUYS_TOPIC_ID:
        print(f"[DEBUG] Ignoring message: Thread ID {getattr(message, 'message_thread_id', 'None')} (not BIGBUYS topic).")
        return

    # Log the received message details
    sender = message.sender_chat if message.sender_chat else message.from_user
    print(f"[DEBUG] Received message: Chat ID={message.chat_id}, Thread ID={message.message_thread_id}, Sender={sender}")

    # Use text or caption if available
    text = message.text or message.caption or ""
    if not text:
        print("[DEBUG] No text or caption found in message.")
        return

    # Extract the first text_link entity for the buyer's wallet address
    full_wallet = None
    entities = message.entities or message.caption_entities or []
    for entity in entities:
        if entity.type == "text_link" and "gmgn.ai/sol/address/" in entity.url:
            full_wallet = entity.url.split("rLkfkJiz_")[1]
            break

    if not full_wallet:
        print("[DEBUG] No buyer wallet address found in entities.")
        return

    # Extract buyer info from the text
    buyer_match = re.search(r'💵\s*(\S+)\s*Heavy Bought\s*(\d+(?:\.\d+)?)\s*SOL', text)
    if not buyer_match:
        print("[DEBUG] Failed to extract buyer info from text.")
        return
    shortened_address = buyer_match.group(1)
    amount = buyer_match.group(2)

    # Extract token info
    token_match = re.search(r'\$(\S+)(?: \(([^)]+)\))?', text)
    if not token_match:
        print("[DEBUG] Failed to extract token symbol.")
        return
    token_symbol = token_match.group(1)
    token_name = token_match.group(2) if token_match.group(2) else ""

    # Extract token address
    address_match = re.search(r'CA: (\w+)️', text)
    if not address_match:
        print("[DEBUG] Failed to extract token address.")
        return
    token_address = address_match.group(1)

    # Extract market data lines
    percentages = re.search(r'(📈 .*?)(?=\n|$)', text).group(1) if re.search(r'(📈 .*?)(?=\n|$)', text) else 'N/A'
    txs_vol = re.search(r'(🎲 .*?)(?=\n|$)', text).group(1) if re.search(r'(🎲 .*?)(?=\n|$)', text) else 'N/A'
    mcp = re.search(r'(💡 .*?)(?=\n|$)', text).group(1) if re.search(r'(💡 .*?)(?=\n|$)', text) else 'N/A'
    liq = re.search(r'(💧 .*?)(?=\n|$)', text).group(1) if re.search(r'(💧 .*?)(?=\n|$)', text) else 'N/A'
    holder = re.search(r'(👥 .*?)(?=\n|$)', text).group(1) if re.search(r'(👥 .*?)(?=\n|$)', text) else 'N/A'
    status = re.search(r'(✅ NoMint / ✅Blacklist / ✅Burnt)', text).group(1) if re.search(r'(✅ NoMint / ✅Blacklist / ✅Burnt)', text) else 'N/A'
    top10 = re.search(r'(✅TOP 10: \d+\.\d+%)', text).group(1) if re.search(r'(✅TOP 10: \d+\.\d+%)', text) else 'N/A'

    # Fetch buyer Twitter username if available
    try:
        url = f"https://gmgn.ai/defi/quotation/v1/smartmoney/sol/walletNew/{full_wallet}"
        response = requests.get(url, timeout=10)
        response.raise_for_status()
        data = response.json()
        if data.get('code') == 0 and 'twitter_username' in data['data'] and data['data']['twitter_username']:
            buyer_display = f"@{data['data']['twitter_username']}"
        else:
            buyer_display = shortened_address
    except Exception as e:
        print(f"[DEBUG] Error fetching buyer details: {e}")
        buyer_display = shortened_address

    # Format token display
    token_display = f"${token_symbol}"
    if token_name:
        token_display += f"({token_name})"

    # Format the message
    message_text = f"💵 {buyer_display} Heavy Bought {token_display} with {amount} SOL\n"
    message_text += f"{percentages}\n"
    message_text += f"{mcp}\n"
    message_text += f"{liq}\n"
    message_text += f"{holder}\n"
    message_text += f"{status}\n"
    message_text += f"{top10}\n"
    message_text += f"🎯DYOR: https://gmgn.ai/sol/token/rLkfkJiz_{token_address}"

    # Check message length and omit TXs/Vol if necessary
    if len(message_text) > 280:
        print(f"[DEBUG] Message too long ({len(message_text)} chars), omitting TXs/Vol.")
        message_text = f"💵 {buyer_display} aped {token_display} with {amount} SOL\n"
        message_text += f"{mcp}\n"
        message_text += f"{liq}\n"
        message_text += f"{holder}\n"
        message_text += f"{top10}\n"
        message_text += f"🎯DYOR: https://gmgn.ai/sol/token/rLkfkJiz_{token_address}"
        if len(message_text) > 280:
            print(f"[DEBUG] Still too long ({len(message_text)} chars), truncating.")
            message_text = message_text[:277] + "..."

    # Forward photo if present and send message to all TARGET_CHAT_IDS + ALWAYS_FORWARD_CHANNEL
    recipient_chats = list(TARGET_CHAT_IDS)
    for chat_id in recipient_chats:
        if chat_id != GROUP_ID:  # Skip the source group
            try:
                if message.photo:
                    photo_file_id = message.photo[-1].file_id
                    await context.bot.send_photo(
                        chat_id=chat_id,
                        photo=photo_file_id,
                        caption=message_text
                    )
                    print(f"[DEBUG] Sent photo with caption to {chat_id}")
                else:
                    await context.bot.send_message(
                        chat_id=chat_id,
                        text=message_text,
                        disable_web_page_preview=True
                    )
                    print(f"[DEBUG] Sent message to {chat_id}: {message_text}")
            except Exception as e:
                print(f"[DEBUG] Failed to send to {chat_id}: {e}")

def main():
    """Start the Telegram bot and run the message and command handlers."""
    application = Application.builder().token(BOT_TOKEN).build()
    application.add_handler(CommandHandler("start", start))
    application.add_handler(CommandHandler("stop", stop))
    application.add_handler(ChatMemberHandler(chat_member_update, ChatMemberHandler.MY_CHAT_MEMBER))
    application.add_handler(MessageHandler(filters.ALL & ~filters.COMMAND, handle_message))
    print("[DEBUG] Starting bot...")
    application.run_polling()

if __name__ == '__main__':
    main()
