import requests
import json
import sqlite3
import time
from telegram import Update, InlineKeyboardButton, InlineKeyboardMarkup
from telegram.ext import Application, CommandHandler, CallbackContext
from telegram.error import RetryAfter

# Configuration
BOT_TOKEN = "7734813990:AAHbWE1K-DWXn7sEXh6XR-k5XYAAseP9lKo"  # Replace with your Telegram bot token
GROUP_ID = "-1002134751475"  # Your Telegram group ID
THREAD_ID_PANCAKE_DEGENZONE = "2049052"  # Replace with actual thread IDs
THREAD_ID_BSC_SPRING_DEGENZONE = "2049052"
THREAD_ID_TRENDING_BSC = "2049052"
REFERRAL_LINK = "https://gmgn.ai/bsc/token/rLkfkJiz_{token_address}"
BEARER_TOKEN = "eyJhbGciOiJIUzI1NiIsInR5cCI6IkpXVCJ9.eyJ1c2VySWQiOiIxNzQzODY0ODUyIiwiaXNBY3RpdmUiOnRydWUsImlhdCI6MTc0MjU3NDU1MywiZXhwIjoxNzQ1MTY2NTUzfQ.d75sRsdgUgz7oRqhd6mmloT7JueRX1OGLgmal6bK6Vs"

# Database setup
conn = sqlite3.connect('bsc_crypto_bot.db', check_same_thread=False)
c = conn.cursor()
c.execute('''CREATE TABLE IF NOT EXISTS posted_tokens (token_address TEXT PRIMARY KEY)''')
c.execute('''CREATE TABLE IF NOT EXISTS alert_settings (chat_id TEXT, module TEXT, enabled INTEGER, PRIMARY KEY (chat_id, module))''')
conn.commit()

# API Fetching Functions
def fetch_api(url, headers=None):
    print(f"[DEBUG] Fetching API: {url}")
    try:
        response = requests.get(url, headers=headers, timeout=10)
        if response.status_code == 200:
            print(f"[DEBUG] API fetch successful for {url}")
            return response.json()
        else:
            print(f"[DEBUG] API fetch failed for {url} with status code: {response.status_code}")
            return None
    except Exception as e:
        print(f"[DEBUG] Exception during API fetch for {url}: {e}")
        return None

def get_pancake_degenzone():
    url = "https://api-fe.mevx.io/api/notifications/public?offset=0&limit=100&notificationType=newSignalBSCPancake"
    headers = {"authorization": f"Bearer {BEARER_TOKEN}"}
    return fetch_api(url, headers)

def get_bsc_spring_degenzone():
    url = "https://api-fe.mevx.io/api/notifications/public?offset=0&limit=100&notificationType=newSignalBSCSpring"
    headers = {"authorization": f"Bearer {BEARER_TOKEN}"}
    return fetch_api(url, headers)

def get_trending_bsc():
    url = "https://api.mevx.io/api/v1/pools?chain=bsc&limit=100&offset=0&baseToken[neq]=0xbb4CdB9CBd36B01bD1cBaEBF2De08d9173bc095c&baseToken[neq]=0x55d398326f99059fF775485246999027B3197955&baseToken[neq]=0x8AC76a51cc950d9822D68b83fE1Ad97B32Cd580d&marketCap[lte]=1000000000&interval=24h&orderBy=tradeCount desc&updatedAt[gte]=1742562000&context=trending"
    return fetch_api(url)

def get_token_audit(token_address):
    url = f"https://api-fe.mevx.io/api/token/audit/bsc/{token_address}"
    return fetch_api(url)

# Security Check Function
def passes_security_check(audit_data):
    if audit_data and 'data' in audit_data:
        goplus = audit_data['data'].get('goplus', {})
        quickintel = audit_data['data'].get('quickIntel', {})
        
        is_verified = goplus.get('isVerified', False) or quickintel.get('isVerified', False)
        if not is_verified:
            print("[DEBUG] Token failed security check: Contract not verified")
            return False
        
        is_honeypot = goplus.get('isHoneypot', False) or quickintel.get('isHoneypot', False)
        if is_honeypot:
            print("[DEBUG] Token failed security check: Identified as honeypot")
            return False
        
        is_proxy = goplus.get('isProxy', False)
        if is_proxy:
            print("[DEBUG] Token failed security check: Identified as proxy")
            return False
        
        return True
    print("[DEBUG] Token failed security check: No valid audit data")
    return False

# Helper Functions
def format_number(num):
    if num >= 1_000_000:
        return f"{num / 1_000_000:.2f}M"
    elif num >= 1_000:
        return f"{num / 1_000:.2f}k"
    else:
        return f"{num:.2f}"

def safe_float(value):
    if isinstance(value, str):
        try:
            return float(value)
        except ValueError:
            return 0.0
    elif isinstance(value, (int, float)):
        return float(value)
    return 0.0

def is_valid_image_url(url):
    try:
        response = requests.head(url, timeout=5)
        content_type = response.headers.get('Content-Type', '')
        return content_type.startswith('image/')
    except Exception as e:
        print(f"[DEBUG] Error checking image URL {url}: {e}")
        return False

# Formatting Functions with Social Links
def format_notification(notification, zone_name):
    message = notification.get('message')
    if isinstance(message, str):
        try:
            data = json.loads(message)
        except json.JSONDecodeError as e:
            print(f"[DEBUG] Error decoding JSON for notification: {e}")
            return None, None, None
    elif isinstance(message, dict):
        data = message
    else:
        print(f"[DEBUG] Unexpected type for 'message': {type(message)}")
        return None, None, None

    name = data.get('name', 'Unknown')
    symbol = data.get('symbol', 'Unknown')
    address = data.get('baseToken', 'N/A')
    link = REFERRAL_LINK.format(token_address=address)
    market_cap = format_number(safe_float(data.get('marketCap', 0)))
    liquidity = format_number(safe_float(data.get('liquidity', 0)))
    burn_percent = safe_float(data.get('burnPercent', 0))
    top10_percent = safe_float(data.get('top10HolderPercent', 0))
    ca = data.get('baseToken', address)
    url_info = data.get('urlInfo', {})
    website = url_info.get('website', '')
    twitter = url_info.get('twitter', '')
    telegram = url_info.get('telegram', '')  # Check for Telegram link
    image_url = url_info.get('image', '')

    formatted_message = (
        f"🔔 **{zone_name} - New Signal**\n\n"
        f"🚀 **Token:** [{name} ({symbol})]({link})\n"
        f"💰 **Market Cap:** ${market_cap}\n"
        f"🌊 **Liquidity:** ${liquidity}\n"
        f"🔥 **Burn Percent:** {burn_percent:.2f}%\n"
        f"🏆 **Top 10 Holders:** {top10_percent:.2f}%\n"
        f"🔗 **CA:** `{ca}`\n"
        f"🏢 **Factory:** {data.get('exchange', 'N/A')}\n"  # Added factory information
    )
    if website:
        formatted_message += f"🌐 **Website:** [Link]({website})\n"
    if twitter:
        formatted_message += f"🐦 **Twitter:** [Link]({twitter})\n"
    if telegram:
        formatted_message += f"📞 **Telegram:** [Link]({telegram})\n"
    return formatted_message, address, image_url

def format_pool(pool, audit_data, zone_name):
    name = pool['baseTokenInfo']['name']
    symbol = pool['baseTokenInfo']['symbol']
    address = pool['baseToken']
    link = REFERRAL_LINK.format(token_address=address)
    market_cap = format_number(safe_float(pool.get('marketCap', 0)))
    price = safe_float(pool.get('priceUsd', 0))
    liquidity = format_number(safe_float(pool.get('liquidUsd', 0)))
    burn_percent = safe_float(pool.get('lpBurnedPercent', 0))
    top10_percent = safe_float(pool['baseTokenInfo'].get('top10HoldersPercent', 0))
    ca = pool.get('baseToken', address)
    socials = pool.get('socials', {})
    website = socials.get('website', '')
    twitter = socials.get('twitter', '')
    telegram = socials.get('telegram', '')  # Check for Telegram link
    image_url = pool['baseTokenInfo'].get('logoUri', '')
    
    liquid_change = safe_float(pool.get('liquidChangePercent', 0))
    buy_volume_24h = safe_float(pool['poolReports'][3]['buyVolume']) if len(pool['poolReports']) > 3 else 0
    sell_volume_24h = safe_float(pool['poolReports'][3]['sellVolume']) if len(pool['poolReports']) > 3 else 0
    trade_count_24h = pool['poolReports'][3]['buyCount'] + pool['poolReports'][3]['sellCount'] if len(pool['poolReports']) > 3 else 0
    
    security_info = ""
    if audit_data and 'data' in audit_data:
        goplus = audit_data['data'].get('goplus', {})
        security_info += "🔒 **Security:**\n"
        security_info += f"  - Verified: {'✅' if goplus.get('isVerified', False) else '❌'}\n"
        security_info += f"  - Honeypot: {'✅' if not goplus.get('isHoneypot', False) else '❌'}\n"
        security_info += f"  - Proxy: {'✅' if not goplus.get('isProxy', False) else '❌'}\n"
    
    pre_factory = pool.get('preFactory', 'N/A')
    factory = pool.get('factory', 'N/A')
    
    formatted_message = (
        f"🚀 **{zone_name}**\n\n"
        f"🚀 **Token:** [{name} ({symbol})]({link})\n"
        f"💰 **Market Cap:** ${market_cap}\n"
        f"💸 **Price:** ${price:.8f}\n"
        f"🌊 **Liquidity:** ${liquidity}\n"
        f"📈 **Liquidity Change (24h):** {liquid_change:.2f}%\n"
        f"💹 **Buy Volume (24h):** ${format_number(buy_volume_24h)}\n"
        f"💱 **Sell Volume (24h):** ${format_number(sell_volume_24h)}\n"
        f"🔄 **Trades (24h):** {trade_count_24h}\n"
        f"🔥 **Burn Percent:** {burn_percent:.2f}%\n"
        f"🏆 **Top 10 Holders:** {top10_percent:.2f}%\n"
        f"🔗 **CA:** `{ca}`\n"
        f"{security_info}"
        f"🏭 **Pre-Factory:** {pre_factory}\n"
        f"🏢 **Factory:** {factory}\n"
    )
    if website:
        formatted_message += f"🌐 **Website:** [Link]({website})\n"
    if twitter:
        formatted_message += f"🐦 **Twitter:** [Link]({twitter})\n"
    if telegram:
        formatted_message += f"📞 **Telegram:** [Link]({telegram})\n"
    return formatted_message, address, image_url

def get_token_keyboard(token_address):
    chart_url = f"https://gmgn.ai/bsc/token/rLkfkJiz_{token_address}"  # Updated chart URL format
    buy_url = f"https://t.me/GMGN_bsc_bot?start=i_rLkfkJiz_c_{token_address}"
    return InlineKeyboardMarkup([
        [InlineKeyboardButton("CHART 📊", url=chart_url),
         InlineKeyboardButton("BUY 🛒", url=buy_url)]
    ])

def get_thread_id(chat_id, module):
    if chat_id == GROUP_ID:
        return {
            'pancake_degenzone': THREAD_ID_PANCAKE_DEGENZONE,
            'bsc_spring_degenzone': THREAD_ID_BSC_SPRING_DEGENZONE,
            'trending_bsc': THREAD_ID_TRENDING_BSC,
        }.get(module)
    return None

def get_enabled_chats(module):
    c.execute("SELECT chat_id FROM alert_settings WHERE module=? AND enabled=1", (module,))
    return [row[0] for row in c.fetchall()]

def enable_all_modules(chat_id):
    modules = ['pancake_degenzone', 'bsc_spring_degenzone', 'trending_bsc']
    for module in modules:
        c.execute("INSERT OR REPLACE INTO alert_settings (chat_id, module, enabled) VALUES (?, ?, 1)", (chat_id, module))
    conn.commit()

# Periodic Update Functions
async def check_pancake_degenzone(bot):
    print("[DEBUG] Starting check_pancake_degenzone")
    data = get_pancake_degenzone()
    notifications = data.get('data', {}).get('notifications', []) if data else []
    print(f"[DEBUG] Processing {len(notifications)} notifications in PancakeSwap DEGENZONE")
    chat_ids = get_enabled_chats('pancake_degenzone')
    for notification in notifications[:5]:  # Top 5
        nid = notification.get('notificationId')
        if not nid:
            continue
        message = notification.get('message')
        if isinstance(message, str):
            try:
                data = json.loads(message)
            except json.JSONDecodeError as e:
                print(f"[DEBUG] Error decoding JSON for notification {nid}: {e}")
                continue
        elif isinstance(message, dict):
            data = message
        else:
            print(f"[DEBUG] Unexpected type for 'message' in notification {nid}")
            continue
        address = data.get('baseToken', 'N/A')
        if not address:
            continue
        print(f"[DEBUG] Token address for notification {nid}: {address}")
        c.execute("SELECT * FROM posted_tokens WHERE token_address=?", (address,))
        if not c.fetchone():
            audit_data = get_token_audit(address)
            if audit_data and passes_security_check(audit_data):
                print(f"[DEBUG] Token {address} passes security check, preparing to send")
                formatted_message, _, image_url = format_notification(notification, "PancakeSwap DEGENZONE")
                keyboard = get_token_keyboard(address)
                for chat_id in chat_ids:
                    thread_id = get_thread_id(chat_id, 'pancake_degenzone')
                    print(f"[DEBUG] Sending message for notification {nid} to chat {chat_id}, thread {thread_id}")
                    try:
                        if image_url and is_valid_image_url(image_url):
                            await bot.send_photo(
                                chat_id=chat_id,
                                message_thread_id=thread_id,
                                photo=image_url,
                                caption=formatted_message,
                                parse_mode='Markdown',
                                reply_markup=keyboard
                            )
                        else:
                            await bot.send_message(
                                chat_id=chat_id,
                                message_thread_id=thread_id,
                                text=formatted_message,
                                parse_mode='Markdown',
                                reply_markup=keyboard,
                                disable_web_page_preview=True
                            )
                        print(f"[DEBUG] Successfully sent message for notification {nid}")
                        c.execute("INSERT INTO posted_tokens VALUES (?)", (address,))
                        conn.commit()
                    except RetryAfter as e:
                        print(f"[DEBUG] Flood control exceeded for notification {nid}. Retrying in {e.retry_after} seconds.")
                        time.sleep(e.retry_after)
                        await bot.send_message(chat_id=chat_id, message_thread_id=thread_id, text=formatted_message,
                                              parse_mode='Markdown', reply_markup=keyboard, disable_web_page_preview=True)
                        c.execute("INSERT INTO posted_tokens VALUES (?)", (address,))
                        conn.commit()
                    except Exception as e:
                        print(f"[DEBUG] Error sending message for notification {nid}: {e}")
            else:
                print(f"[DEBUG] Token {address} failed security check or audit fetch failed")
        else:
            print(f"[DEBUG] Token {address} already posted, skipping.")
        time.sleep(1)

async def check_bsc_spring_degenzone(bot):
    print("[DEBUG] Starting check_bsc_spring_degenzone")
    data = get_bsc_spring_degenzone()
    notifications = data.get('data', {}).get('notifications', []) if data else []
    print(f"[DEBUG] Processing {len(notifications)} notifications in BSC SPRING DEGEN ZONE")
    chat_ids = get_enabled_chats('bsc_spring_degenzone')
    for notification in notifications[:5]:  # Top 5
        nid = notification.get('notificationId')
        if not nid:
            continue
        message = notification.get('message')
        if isinstance(message, str):
            try:
                data = json.loads(message)
            except json.JSONDecodeError as e:
                print(f"[DEBUG] Error decoding JSON for notification {nid}: {e}")
                continue
        elif isinstance(message, dict):
            data = message
        else:
            print(f"[DEBUG] Unexpected type for 'message' in notification {nid}")
            continue
        address = data.get('baseToken', 'N/A')
        if not address:
            continue
        print(f"[DEBUG] Token address for notification {nid}: {address}")
        c.execute("SELECT * FROM posted_tokens WHERE token_address=?", (address,))
        if not c.fetchone():
            audit_data = get_token_audit(address)
            if audit_data and passes_security_check(audit_data):
                print(f"[DEBUG] Token {address} passes security check, preparing to send")
                formatted_message, _, image_url = format_notification(notification, "BSC SPRING DEGEN ZONE")
                keyboard = get_token_keyboard(address)
                for chat_id in chat_ids:
                    thread_id = get_thread_id(chat_id, 'bsc_spring_degenzone')
                    print(f"[DEBUG] Sending message for notification {nid} to chat {chat_id}, thread {thread_id}")
                    try:
                        if image_url and is_valid_image_url(image_url):
                            await bot.send_photo(
                                chat_id=chat_id,
                                message_thread_id=thread_id,
                                photo=image_url,
                                caption=formatted_message,
                                parse_mode='Markdown',
                                reply_markup=keyboard
                            )
                        else:
                            await bot.send_message(
                                chat_id=chat_id,
                                message_thread_id=thread_id,
                                text=formatted_message,
                                parse_mode='Markdown',
                                reply_markup=keyboard,
                                disable_web_page_preview=True
                            )
                        print(f"[DEBUG] Successfully sent message for notification {nid}")
                        c.execute("INSERT INTO posted_tokens VALUES (?)", (address,))
                        conn.commit()
                    except RetryAfter as e:
                        print(f"[DEBUG] Flood control exceeded for notification {nid}. Retrying in {e.retry_after} seconds.")
                        time.sleep(e.retry_after)
                        await bot.send_message(chat_id=chat_id, message_thread_id=thread_id, text=formatted_message,
                                              parse_mode='Markdown', reply_markup=keyboard, disable_web_page_preview=True)
                        c.execute("INSERT INTO posted_tokens VALUES (?)", (address,))
                        conn.commit()
                    except Exception as e:
                        print(f"[DEBUG] Error sending message for notification {nid}: {e}")
            else:
                print(f"[DEBUG] Token {address} failed security check or audit fetch failed")
        else:
            print(f"[DEBUG] Token {address} already posted, skipping.")
        time.sleep(1)

async def check_trending_bsc(bot):
    print("[DEBUG] Starting check_trending_bsc")
    data = get_trending_bsc()
    if data and 'pools' in data:
        pools = data['pools'][:10]  # Changed to top 10 trending pools
        print(f"[DEBUG] Processing {len(pools)} pools in TRENDING BSC")
        chat_ids = get_enabled_chats('trending_bsc')
        for pool in pools:
            address = pool['baseToken']
            print(f"[DEBUG] Checking token {address} in TRENDING BSC")
            c.execute("SELECT * FROM posted_tokens WHERE token_address=?", (address,))
            if not c.fetchone():
                audit_data = get_token_audit(address)
                if audit_data and passes_security_check(audit_data):
                    print(f"[DEBUG] Token {address} passes security check, preparing to send")
                    message, address, image_url = format_pool(pool, audit_data, "TRENDING BSC")
                    keyboard = get_token_keyboard(address)
                    for chat_id in chat_ids:
                        thread_id = get_thread_id(chat_id, 'trending_bsc')
                        print(f"[DEBUG] Sending message for token {address} to chat {chat_id}, thread {thread_id}")
                        try:
                            if image_url and is_valid_image_url(image_url):
                                await bot.send_photo(
                                    chat_id=chat_id,
                                    message_thread_id=thread_id,
                                    photo=image_url,
                                    caption=message,
                                    parse_mode='Markdown',
                                    reply_markup=keyboard
                                )
                            else:
                                await bot.send_message(
                                    chat_id=chat_id,  # Fixed typo from chart_id
                                    message_thread_id=thread_id,
                                    text=message,
                                    parse_mode='Markdown',
                                    reply_markup=keyboard,
                                    disable_web_page_preview=True
                                )
                            print(f"[DEBUG] Successfully sent message for token {address}")
                            c.execute("INSERT INTO posted_tokens VALUES (?)", (address,))
                            conn.commit()
                        except RetryAfter as e:
                            print(f"[DEBUG] Flood control exceeded for token {address}. Retrying in {e.retry_after} seconds.")
                            time.sleep(e.retry_after)
                            await bot.send_message(chat_id=chat_id, message_thread_id=thread_id, text=message,
                                                  parse_mode='Markdown', reply_markup=keyboard, disable_web_page_preview=True)
                            c.execute("INSERT INTO posted_tokens VALUES (?)", (address,))
                            conn.commit()
                        except Exception as e:
                            print(f"[DEBUG] Error sending message for token {address}: {e}")
                else:
                    print(f"[DEBUG] Token {address} failed security check or audit fetch failed")
            else:
                print(f"[DEBUG] Token {address} already posted, skipping.")
            time.sleep(1)

# Command Handlers
async def start(update: Update, context: CallbackContext):
    chat_id = str(update.message.chat.id)
    enable_all_modules(chat_id)
    message = (
        "All BSC alerts enabled. To disable a specific module, use /stop [module].\n"
        "Available modules: pancake_degenzone, bsc_spring_degenzone, trending_bsc\n"
        "Example: /stop pancake_degenzone\n\n"
        "Join our main group @vaultboyportal for more updates!"
    )
    await update.message.reply_text(message)

# Scheduler Function
async def schedule_main_checks(context: CallbackContext):
    print("[DEBUG] Running schedule_main_checks")
    try:
        bot = context.bot
        await check_pancake_degenzone(bot)
        await check_bsc_spring_degenzone(bot)
        await check_trending_bsc(bot)
    except Exception as e:
        print(f"[DEBUG] Error in schedule_main_checks: {e}")

# Main Function
def main():
    print("[DEBUG] Initializing BSC bot")
    application = Application.builder().token(BOT_TOKEN).build()

    application.add_handler(CommandHandler("start", start))

    job_queue = application.job_queue
    print("[DEBUG] Setting up job queue")
    job_queue.run_repeating(schedule_main_checks, interval=60, first=0)

    print("[DEBUG] Starting bot polling")
    application.run_polling()

if __name__ == "__main__":
    main()
