import re
import requests
from telegram import Update
from telegram.ext import Application, MessageHandler, CommandHandler, ChatMemberHandler, filters

# Bot configuration
BOT_TOKEN = '7224996665:AAHjOzODEvWpRaMiGEZsIWtG9ebdyfjsoz8'  # Your bot token
GROUP_ID = -1002596386377  # Your source group ID
KOLFOMO_TOPIC_ID = 882  # Your KOLFOMO topic ID

# File to store subscribed chat IDs
SUBSCRIBED_CHATS_FILE = 'subscribed_chats.txt'

# Always forward to this channel (even if not subscribed)
ALWAYS_FORWARD_CHANNEL = -1002264845133

# Functions to manage chat IDs in file
def load_subscribed_chats():
    """Load chat IDs from file into a set."""
    try:
        with open(SUBSCRIBED_CHATS_FILE, 'r') as f:
            return set(int(line.strip()) for line in f if line.strip())
    except FileNotFoundError:
        return set()

def save_subscribed_chats(chat_ids):
    """Save chat IDs from set to file."""
    with open(SUBSCRIBED_CHATS_FILE, 'w') as f:
        for chat_id in chat_ids:
            f.write(f"{chat_id}\n")

# Initialize set of chat IDs where bot will post
TARGET_CHAT_IDS = load_subscribed_chats()

# Command to subscribe a chat
async def start(update: Update, context):
    """Add chat to subscribers when /start is called, unless it's the source group."""
    chat_id = update.message.chat_id
    if chat_id == GROUP_ID:
        await context.bot.send_message(chat_id=chat_id, text="This is the source group. Bot cannot subscribe here.")
        return
    if chat_id not in TARGET_CHAT_IDS:
        TARGET_CHAT_IDS.add(chat_id)
        save_subscribed_chats(TARGET_CHAT_IDS)
        await context.bot.send_message(chat_id=chat_id, text="Bot will now post updates here.")
    else:
        await context.bot.send_message(chat_id=chat_id, text="Bot is already subscribed to this chat.")

# Command to unsubscribe a chat
async def stop(update: Update, context):
    """Remove chat from subscribers when /stop is called."""
    chat_id = update.message.chat_id
    if chat_id in TARGET_CHAT_IDS:
        TARGET_CHAT_IDS.remove(chat_id)
        save_subscribed_chats(TARGET_CHAT_IDS)
        await context.bot.send_message(chat_id=chat_id, text="Bot will no longer post updates here.")
    else:
        await context.bot.send_message(chat_id=chat_id, text="Bot is not subscribed to this chat.")

# Handle bot status changes (e.g., removal from group)
async def chat_member_update(update: Update, context):
    """Remove chat ID from subscribers if bot is kicked or leaves."""
    my_chat_member = update.my_chat_member
    if my_chat_member:
        new_status = my_chat_member.new_chat_member.status
        chat_id = my_chat_member.chat.id
        if new_status in ['kicked', 'left']:
            if chat_id in TARGET_CHAT_IDS:
                TARGET_CHAT_IDS.remove(chat_id)
                save_subscribed_chats(TARGET_CHAT_IDS)
                print(f"[DEBUG] Removed chat ID {chat_id} from subscribed chats.")

# Handle messages and group migrations
async def handle_message(update: Update, context):
    """Process messages from the source group and handle group migrations."""
    message = update.message
    if not message:
        print("[DEBUG] Update received but no message found.")
        return

    # Handle group migration to supergroup
    if message.migrate_to_chat_id:
        old_chat_id = message.chat_id
        new_chat_id = message.migrate_to_chat_id
        if old_chat_id in TARGET_CHAT_IDS:
            TARGET_CHAT_IDS.remove(old_chat_id)
            TARGET_CHAT_IDS.add(new_chat_id)
            save_subscribed_chats(TARGET_CHAT_IDS)
            print(f"[DEBUG] Updated chat ID from {old_chat_id} to {new_chat_id}.")
        return

    # Check if message is from the source group and topic
    if message.chat_id != GROUP_ID:
        print(f"[DEBUG] Ignoring message from chat ID {message.chat_id} (not source group).")
        return
    if not hasattr(message, 'message_thread_id') or message.message_thread_id != KOLFOMO_TOPIC_ID:
        print(f"[DEBUG] Ignoring message: Thread ID {getattr(message, 'message_thread_id', 'None')} (not KOLFOMO topic).")
        return

    # Log the received message details
    sender = message.sender_chat if message.sender_chat else message.from_user
    print(f"[DEBUG] Received message: Chat ID={message.chat_id}, Thread ID={message.message_thread_id}, Sender={sender}")

    text = message.text or message.caption or ""
    if not text:
        print("[DEBUG] No text or caption found in message.")
        return

    # Extract number of KOLs and token name
    kol_token_match = re.search(r'(\d+) KOL Buy (\S+)', text)
    print(f"[DEBUG] KOL Token Match: {kol_token_match.group() if kol_token_match else 'None'}")
    if not kol_token_match:
        print("[DEBUG] Failed to extract number of KOLs or token name.")
        return
    num_kols = int(kol_token_match.group(1))
    token_name = kol_token_match.group(2)

    # Extract token contract
    contract_match = re.search(r'CA: (\w+)', text)
    print(f"[DEBUG] Contract Match: {contract_match.group(1) if contract_match else 'None'}")
    if not contract_match:
        print("[DEBUG] Failed to extract token contract.")
        return
    token_contract = contract_match.group(1)

    # Extract KOL wallet addresses from message entities
    kol_wallets = []
    entities = message.entities or message.caption_entities or []
    print(f"[DEBUG] Total entities found: {len(entities)}")
    for entity in entities:
        print(f"[DEBUG] Entity type: {entity.type}, URL: {entity.url if entity.type in ['text_link', 'url'] else 'N/A'}")
        if (entity.type == "text_link" or entity.type == "url") and "gmgn.ai/sol/address/" in entity.url:
            url_parts = entity.url.split("rLkfkJiz_")
            if len(url_parts) > 1:
                wallet_address = url_parts[1].split("?")[0].split("/")[0]
                kol_wallets.append(wallet_address)
    print(f"[DEBUG] Extracted KOL wallets: {kol_wallets}")
    if not kol_wallets:
        print("[DEBUG] No KOL wallet addresses found.")
        return

    # Extract market data
    mcp_match = re.search(r'💡 MCP: \$(\d+\.\d+[KM]?)', text)
    mcp = mcp_match.group(1) if mcp_match else 'N/A'
    print(f"[DEBUG] MCP Match: {mcp}")

    liq_match = re.search(r'💧 Liq: (.*?)(?=\n|$)', text)
    liq = liq_match.group(1).strip() if liq_match else 'N/A'
    print(f"[DEBUG] Liq Match: {liq}")

    holder_match = re.search(r'👥 Holder: (\d+)', text)
    holder = holder_match.group(1) if holder_match else 'N/A'
    print(f"[DEBUG] Holder Match: {holder}")

    # Fetch KOL Twitter usernames from the API
    kol_usernames = []
    for wallet in kol_wallets:
        try:
            url = f"https://gmgn.ai/defi/quotation/v1/smartmoney/sol/walletNew/{wallet}"
            response = requests.get(url, timeout=10)
            response.raise_for_status()
            data = response.json()
            if data.get('code') == 0 and 'twitter_username' in data['data'] and data['data']['twitter_username']:
                twitter_username = data['data']['twitter_username']
                kol_usernames.append(f"@{twitter_username}")
                print(f"[DEBUG] Fetched KOL username: @{twitter_username}")
            else:
                print(f"[DEBUG] Skipping wallet {wallet}: No Twitter username found.")
        except requests.RequestException as e:
            print(f"[DEBUG] Error fetching KOL details for {wallet}: {e}")
        except (KeyError, ValueError) as e:
            print(f"[DEBUG] Error parsing API response for {wallet}: {e}")

    # Format the message
    message_text = f"📣JUST NOW: {num_kols} KOLs aped #{token_name}!! 💎\n"
    message_text += f"💡 MCP: ${mcp}\n"
    message_text += f"💧 Liq: {liq}\n"
    message_text += f"👥 Holders: {holder}\n"
    message_text += f"https://gmgn.ai/sol/token/rLkfkJiz_{token_contract}\n"
    message_text += f"KOLS: {','.join(kol_usernames)}"

    # Check message length
    print(f"[DEBUG] Message length: {len(message_text)} characters")
    if len(message_text) > 280:
        print(f"[DEBUG] Message too long ({len(message_text)} chars), truncating.")
        message_text = message_text[:277] + "..."

    # Forward photo if present and send message to all TARGET_CHAT_IDS + ALWAYS_FORWARD_CHANNEL
    recipient_chats = list(TARGET_CHAT_IDS)
    for chat_id in recipient_chats:
        if chat_id != GROUP_ID:  # Skip the source group
            try:
                    await context.bot.send_message(
                        chat_id=chat_id,
                        text=message_text,
                        disable_web_page_preview=True
                    )
                    print(f"[DEBUG] Sent message to {chat_id}: {message_text}")
            except Exception as e:
                print(f"[DEBUG] Failed to send to {chat_id}: {e}")

def main():
    """Start the Telegram bot and run the message and command handlers."""
    application = Application.builder().token(BOT_TOKEN).build()
    application.add_handler(CommandHandler("start", start))
    application.add_handler(CommandHandler("stop", stop))
    application.add_handler(ChatMemberHandler(chat_member_update, ChatMemberHandler.MY_CHAT_MEMBER))
    application.add_handler(MessageHandler(filters.ALL & ~filters.COMMAND, handle_message))
    print("[DEBUG] Starting bot...")
    application.run_polling()

if __name__ == '__main__':
    main()
