import requests
import json
import sqlite3
import time
from telegram import Update, InlineKeyboardButton, InlineKeyboardMarkup
from telegram.ext import Application, CommandHandler, CallbackContext
from telegram.error import RetryAfter

# Configuration
BOT_TOKEN = "7269427479:AAF48ZMPyI7PIegB_NP4VbRqMVDC3m2fvhw"
GROUP_ID = "-1002134751475"
THREAD_ID_MEME_ZONE = "2049015"
THREAD_ID_TRENDING_PUMP = "2048966"
THREAD_ID_GRADUATED = "2049015"
THREAD_ID_TRENDING_RAY = "2048964"
THREAD_ID_ABOUT_TO_GRADUATE = "2049015"
REFERRAL_LINK = "https://gmgn.ai/sol/token/rLkfkJiz_{token_address}"

# Database setup
conn = sqlite3.connect('crypto_bot.db', check_same_thread=False)
c = conn.cursor()
c.execute('''CREATE TABLE IF NOT EXISTS posted_notifications (notification_id TEXT PRIMARY KEY)''')
c.execute('''CREATE TABLE IF NOT EXISTS posted_tokens (token_address TEXT PRIMARY KEY)''')
c.execute('''CREATE TABLE IF NOT EXISTS alert_settings (chat_id TEXT, module TEXT, enabled INTEGER, PRIMARY KEY (chat_id, module))''')
conn.commit()

# API Fetching Functions
def fetch_api(url, headers=None):
    print(f"[DEBUG] Fetching API: {url}")
    try:
        response = requests.get(url, headers=headers)
        if response.status_code == 200:
            print(f"[DEBUG] API fetch successful for {url}")
            return response.json()
        else:
            print(f"[DEBUG] API fetch failed for {url} with status code: {response.status_code}")
            return None
    except Exception as e:
        print(f"[DEBUG] Exception during API fetch for {url}: {e}")
        return None

def get_meme_zone():
    url = "https://api.mevx.io/api/v1/pools?chain=sol&limit=30&offset=0&exchange=pump&exchange=raydium&exchange=moonshot&mintable[eq]=false&freezable[eq]=false&baseToken[neq]=So11111111111111111111111111111111111111112&marketCap[lte]=1000000000&interval=24h&orderBy=tradeCount desc&updatedAt[gte]=1742551200&context=trending"
    return fetch_api(url)

def get_trending_pump():
    url = "https://api-fe.mevx.io/api/notifications/public?offset=0&limit=30&notificationType=newSignal"
    headers = {
        "authorization": "Bearer eyJhbGciOiJIUzI1NiIsInR5cCI6IkpXVCJ9.eyJ1c2VySWQiOiIxNzQzODY0ODUyIiwiaXNBY3RpdmUiOnRydWUsImlhdCI6MTc0MjU3NDU1MywiZXhwIjoxNzQ1MTY2NTUzfQ.d75sRsdgUgz7oRqhd6mmloT7JueRX1OGLgmal6bK6Vs"
    }
    return fetch_api(url, headers)

def get_trending_ray():
    url = "https://api-fe.mevx.io/api/notifications/public?offset=0&limit=30&notificationType=newSignalRay"
    headers = {
        "authorization": "Bearer eyJhbGciOiJIUzI1NiIsInR5cCI6IkpXVCJ9.eyJ1c2VySWQiOiIxNzQzODY0ODUyIiwiaXNBY3RpdmUiOnRydWUsImlhdCI6MTc0MjU3NDU1MywiZXhwIjoxNzQ1MTY2NTUzfQ.d75sRsdgUgz7oRqhd6mmloT7JueRX1OGLgmal6bK6Vs"
    }
    return fetch_api(url, headers)

def get_meme_zone_filtered():
    url = "https://api.mevx.io/trade/memezone?minSocial=1&topHolder=1&prevExchange=pump,moonshot&tokenAgeFrom=1742489538&tokenAgeTo=1742575938&orderBy=listraydium&interval=1440&limit=50"
    return fetch_api(url)

def get_about_to_graduate():
    url = "https://api.mevx.io/trade/memezone?minSocial=1&topHolder=0&exchange=pump,moonshot&tokenAgeFrom=1742489739&tokenAgeTo=1742576139&orderBy=percent&interval=1440&limit=50"
    return fetch_api(url)

# New Function to Validate Image URLs
def is_valid_image_url(url):
    """Check if a URL points to a valid image by examining the Content-Type header."""
    try:
        response = requests.head(url, timeout=5)
        content_type = response.headers.get('Content-Type', '')
        return content_type.startswith('image/')
    except Exception as e:
        print(f"[DEBUG] Error checking image URL {url}: {e}")
        return False

# Helper Functions
def format_number(num):
    if num >= 1_000_000:
        return f"{num / 1_000_000:.2f}M"
    elif num >= 1_000:
        return f"{num / 1_000:.2f}k"
    else:
        return f"{num:.2f}"

def safe_float(value):
    if isinstance(value, str):
        try:
            return float(value)
        except ValueError:
            return 0.0
    elif isinstance(value, (int, float)):
        return float(value)
    return 0.0

# Formatting Functions (Updated "Mint" to "CA")
def format_meme_zone(pool):
    name = pool['baseTokenInfo']['name']
    symbol = pool['baseTokenInfo']['symbol']
    address = pool['baseToken']
    link = REFERRAL_LINK.format(token_address=address)
    market_cap = format_number(safe_float(pool.get('marketCap', 0)))
    price = safe_float(pool.get('priceUsd', 0))
    liquidity = format_number(safe_float(pool.get('liquidUsd', 0)))
    holders = pool.get('holder', 'N/A')
    burn_percent = pool.get('lpBurnedPercent', 'N/A')
    top10_percent = safe_float(pool['baseTokenInfo'].get('top10HoldersPercent', 0))
    mint = pool.get('baseToken', address)
    socials = pool.get('socials', {})
    website = socials.get('website', '')
    image_url = pool['baseTokenInfo'].get('logoUri', '')

    formatted_message = (
        f"🚀 **MEME ZONE**\n\n"
        f"🚀 **Token:** [{name} ({symbol})]({link})\n"
        f"💰 **Market Cap:** ${market_cap}\n"
        f"💸 **Price:** ${price:.8f}\n"
        f"🌊 **Liquidity:** ${liquidity}\n"
        f"🔥 **Burn Percent:** {burn_percent}%\n"
        f"👥 **Holders:** {holders}\n"
        f"🏆 **Top 10 Holders:** {top10_percent:.2f}%\n"
        f"🔗 **CA:** `{mint}`\n"
    )
    if website:
        formatted_message += f"🌐 **Website:** [Link]({website})\n"
    return formatted_message, address, image_url

def format_trending(notification, zone_name):
    if isinstance(notification, str):
        try:
            notification = json.loads(notification)
        except json.JSONDecodeError as e:
            print(f"[DEBUG] Error decoding JSON for notification: {e}")
            return None, None, None

    message = notification.get('message')
    if isinstance(message, str):
        try:
            data = json.loads(message)
        except json.JSONDecodeError as e:
            print(f"[DEBUG] Error decoding JSON for message: {e}")
            return None, None, None
    elif isinstance(message, dict):
        data = message
    else:
        print(f"[DEBUG] Unexpected type for 'message': {type(message)}")
        return None, None, None

    name = data.get('name', 'Unknown')
    symbol = data.get('symbol', 'Unknown')
    address = data.get('mint', 'N/A')
    link = REFERRAL_LINK.format(token_address=address)
    market_cap = format_number(safe_float(data.get('marketCap', 0)))
    price = safe_float(data.get('priceUsd', 0))
    liquidity = format_number(safe_float(data.get('liquidity', 0)))
    burn_percent = safe_float(data.get('burnPercent', 0))
    holders = data.get('holder', 'N/A')
    top10_percent = safe_float(data.get('top10HolderPercent', 0))
    mint = data.get('mint', address)
    socials = data.get('socials', {})
    website = socials.get('website', '')
    url_info = data.get('urlInfo', {})
    image_url = url_info.get('image2', '')

    formatted_message = (
        f"🔔 **{zone_name} - New Signal**\n\n"
        f"🚀 **Token:** [{name} ({symbol})]({link})\n"
        f"💰 **Market Cap:** ${market_cap}\n"
        f"💸 **Price:** ${price:.8f}\n"
        f"🌊 **Liquidity:** ${liquidity}\n"
        f"🔥 **Burn Percent:** {burn_percent:.2f}%\n"
        f"👥 **Holders:** {holders}\n"
        f"🏆 **Top 10 Holders:** {top10_percent:.2f}%\n"
        f"🔗 **CA:** `{mint}`\n"
    )
    if website:
        formatted_message += f"🌐 **Website:** [Link]({website})\n"
    return formatted_message, address, image_url

def format_graduated_or_trending(token):
    name = token.get('name', 'Unknown')
    symbol = token.get('symbol', 'Unknown')
    address = token.get('address', 'N/A')
    link = REFERRAL_LINK.format(token_address=address)
    market_cap = format_number(safe_float(token.get('marketCap', 0)))
    price = safe_float(token.get('priceInUSD', 0))
    liquidity = format_number(safe_float(token.get('Liquidity', 0)))
    holders = token.get('holder', 'N/A')
    burn_percent = safe_float(token.get('burnPercent', 0))
    top10_percent = safe_float(token.get('top10HolderPercent', 0))
    mint = token.get('mint', address)
    socials = token.get('socials', {})
    website = socials.get('website', '')
    image_url = token.get('urlInfo', {}).get('image2', '')

    formatted_message = (
        f"🎉 **GRADUATED**\n\n"
        f"🚀 **Token:** [{name} ({symbol})]({link})\n"
        f"💰 **Market Cap:** ${market_cap}\n"
        f"💸 **Price:** ${price:.8f}\n"
        f"🌊 **Liquidity:** ${liquidity}\n"
        f"🔥 **Burn Percent:** {burn_percent:.2f}%\n"
        f"👥 **Holders:** {holders}\n"
        f"🏆 **Top 10 Holders:** {top10_percent:.2f}%\n"
        f"🔗 **CA:** `{mint}`\n"
    )
    if website:
        formatted_message += f"🌐 **Website:** [Link]({website})\n"
    return formatted_message, address, image_url

def format_about_to_graduate(token):
    name = token.get('name', 'Unknown')
    symbol = token.get('symbol', 'Unknown')
    address = token.get('address', 'N/A')
    link = REFERRAL_LINK.format(token_address=address)
    market_cap = format_number(safe_float(token.get('marketCap', 0)))
    price = safe_float(token.get('priceInUSD', 0))
    liquidity = format_number(safe_float(token.get('Liquidity', 0)))
    holders = token.get('holder', 'N/A')
    burn_percent = safe_float(token.get('burnPercent', 0))
    top10_percent = safe_float(token.get('top10HolderPercent', 0))
    mint = token.get('mint', address)
    socials = token.get('socials', {})
    website = socials.get('website', '')
    image_url = token.get('urlInfo', {}).get('image2', '')

    formatted_message = (
        f"🎓 **ABOUT TO GRADUATE**\n\n"
        f"🚀 **Token:** [{name} ({symbol})]({link})\n"
        f"💰 **Market Cap:** ${market_cap}\n"
        f"💸 **Price:** ${price:.8f}\n"
        f"🌊 **Liquidity:** ${liquidity}\n"
        f"🔥 **Burn Percent:** {burn_percent:.2f}%\n"
        f"👥 **Holders:** {holders}\n"
        f"🏆 **Top 10 Holders:** {top10_percent:.2f}%\n"
        f"🔗 **CA:** `{mint}`\n"
    )
    if website:
        formatted_message += f"🌐 **Website:** [Link]({website})\n"
    return formatted_message, address, image_url

def get_token_keyboard(token_address):
    chart_url = f"https://gmgn.ai/sol/token/rLkfkJiz_{token_address}"
    buy_url = f"https://t.me/GMGN_sol_bot?start=i_rLkfkJiz_c_{token_address}"
    return InlineKeyboardMarkup([
        [
            InlineKeyboardButton("CHART 📊", url=chart_url),
            InlineKeyboardButton("BUY 🛒", url=buy_url)
        ]
    ])

def get_thread_id(chat_id, module):
    if chat_id == GROUP_ID:
        return {
            'meme': THREAD_ID_MEME_ZONE,
            'trending_pump': THREAD_ID_TRENDING_PUMP,
            'trending_ray': THREAD_ID_TRENDING_RAY,
            'graduated': THREAD_ID_GRADUATED,
            'bonding': THREAD_ID_ABOUT_TO_GRADUATE
        }.get(module)
    return None

def get_enabled_chats(module):
    c.execute("SELECT chat_id FROM alert_settings WHERE module=? AND enabled=1", (module,))
    return [row[0] for row in c.fetchall()]

def enable_all_modules(chat_id):
    for module in ['meme', 'trending_pump', 'trending_ray', 'graduated', 'bonding']:
        c.execute("INSERT OR REPLACE INTO alert_settings (chat_id, module, enabled) VALUES (?, ?, 1)", (chat_id, module))
    conn.commit()

# Periodic Update Functions with Enhanced Logging and Double Check
async def check_meme_zone(bot):
    print("[DEBUG] Starting check_meme_zone")
    data = get_meme_zone()
    if data and 'pools' in data:
        print(f"[DEBUG] Processing {len(data['pools'])} pools in meme zone")
        chat_ids = get_enabled_chats('meme')
        for chat_id in chat_ids:
            for pool in data['pools'][:5]:
                address = pool['baseToken']
                print(f"[DEBUG] Checking token {address} in meme zone")
                c.execute("SELECT * FROM posted_tokens WHERE token_address=?", (address,))
                if not c.fetchone():
                    print(f"[DEBUG] Token {address} not posted yet, preparing to send")
                    message, address, image_url = format_meme_zone(pool)
                    keyboard = get_token_keyboard(address)
                    thread_id = get_thread_id(chat_id, 'meme')
                    print(f"[DEBUG] Sending message for token {address} to chat {chat_id}, thread {thread_id}")
                    try:
                        if image_url and is_valid_image_url(image_url):
                            await bot.send_photo(
                                chat_id=chat_id,
                                message_thread_id=thread_id,
                                photo=image_url,
                                caption=message,
                                parse_mode='Markdown',
                                reply_markup=keyboard
                            )
                        else:
                            await bot.send_message(
                                chat_id=chat_id,
                                message_thread_id=thread_id,
                                text=message,
                                parse_mode='Markdown',
                                reply_markup=keyboard,
                                disable_web_page_preview=True
                            )
                        print(f"[DEBUG] Successfully sent message for token {address}")
                        c.execute("INSERT INTO posted_tokens VALUES (?)", (address,))
                        conn.commit()
                        print(f"[DEBUG] Inserted token {address} into posted_tokens")
                    except RetryAfter as e:
                        print(f"[DEBUG] Flood control exceeded for token {address}. Retrying in {e.retry_after} seconds.")
                        time.sleep(e.retry_after)
                        if image_url and is_valid_image_url(image_url):
                            await bot.send_photo(
                                chat_id=chat_id,
                                message_thread_id=thread_id,
                                photo=image_url,
                                caption=message,
                                parse_mode='Markdown',
                                reply_markup=keyboard
                            )
                        else:
                            await bot.send_message(
                                chat_id=chat_id,
                                message_thread_id=thread_id,
                                text=message,
                                parse_mode='Markdown',
                                reply_markup=keyboard,
                                disable_web_page_preview=True
                            )
                        print(f"[DEBUG] Successfully sent message for token {address} after retry")
                        c.execute("INSERT INTO posted_tokens VALUES (?)", (address,))
                        conn.commit()
                        print(f"[DEBUG] Inserted token {address} into posted_tokens after retry")
                    except Exception as e:
                        print(f"[DEBUG] Error sending message for token {address} to chat {chat_id}, thread {thread_id}: {e}")
                else:
                    print(f"[DEBUG] Token {address} already posted, skipping.")
                time.sleep(1)

async def check_trending_pump(bot):
    print("[DEBUG] Starting check_trending_pump")
    data = get_trending_pump()
    if data:
        notifications = data.get('data', {}).get('notifications', []) if isinstance(data, dict) else data
        if not notifications:
            print("[DEBUG] No 'notifications' key found or empty")
            return
        print(f"[DEBUG] Processing {len(notifications)} notifications in trending pump")
        chat_ids = get_enabled_chats('trending_pump')
        for notification in notifications:
            nid = notification.get('notificationId')
            if not nid:
                continue
            print(f"[DEBUG] Checking notification {nid}")
            c.execute("SELECT * FROM posted_notifications WHERE notification_id=?", (nid,))
            if not c.fetchone():
                message, address, image_url = format_trending(notification, "TRENDING PUMP")
                if message is None:
                    continue
                print(f"[DEBUG] Token address for notification {nid}: {address}")
                # Double check: skip if token is already posted
                c.execute("SELECT * FROM posted_tokens WHERE token_address=?", (address,))
                if c.fetchone():
                    print(f"[DEBUG] Token {address} already posted, skipping notification {nid}")
                    continue
                keyboard = get_token_keyboard(address)
                for chat_id in chat_ids:
                    thread_id = get_thread_id(chat_id, 'trending_pump')
                    print(f"[DEBUG] Sending message for notification {nid} (token {address}) to chat {chat_id}, thread {thread_id}")
                    try:
                        if image_url and is_valid_image_url(image_url):
                            await bot.send_photo(
                                chat_id=chat_id,
                                message_thread_id=thread_id,
                                photo=image_url,
                                caption=message,
                                parse_mode='Markdown',
                                reply_markup=keyboard
                            )
                        else:
                            await bot.send_message(
                                chat_id=chat_id,
                                message_thread_id=thread_id,
                                text=message,
                                parse_mode='Markdown',
                                reply_markup=keyboard,
                                disable_web_page_preview=True
                            )
                        print(f"[DEBUG] Successfully sent message for notification {nid} (token {address})")
                        c.execute("INSERT INTO posted_notifications VALUES (?)", (nid,))
                        c.execute("INSERT OR IGNORE INTO posted_tokens VALUES (?)", (address,))
                        conn.commit()
                        print(f"[DEBUG] Inserted notification {nid} and token {address} into database")
                    except RetryAfter as e:
                        print(f"[DEBUG] Flood control exceeded for notification {nid} (token {address}). Retrying in {e.retry_after} seconds.")
                        time.sleep(e.retry_after)
                        if image_url and is_valid_image_url(image_url):
                            await bot.send_photo(
                                chat_id=chat_id,
                                message_thread_id=thread_id,
                                photo=image_url,
                                caption=message,
                                parse_mode='Markdown',
                                reply_markup=keyboard
                            )
                        else:
                            await bot.send_message(
                                chat_id=chat_id,
                                message_thread_id=thread_id,
                                text=message,
                                parse_mode='Markdown',
                                reply_markup=keyboard,
                                disable_web_page_preview=True
                            )
                        print(f"[DEBUG] Successfully sent message for notification {nid} (token {address}) after retry")
                        c.execute("INSERT INTO posted_notifications VALUES (?)", (nid,))
                        c.execute("INSERT OR IGNORE INTO posted_tokens VALUES (?)", (address,))
                        conn.commit()
                        print(f"[DEBUG] Inserted notification {nid} and token {address} into database after retry")
                    except Exception as e:
                        print(f"[DEBUG] Error sending message for notification {nid} (token {address}) to chat {chat_id}, thread {thread_id}: {e}")
                time.sleep(1)
            else:
                print(f"[DEBUG] Notification {nid} already posted, skipping.")

async def check_trending_ray(bot):
    print("[DEBUG] Starting check_trending_ray")
    data = get_trending_ray()
    if data:
        notifications = data.get('data', {}).get('notifications', []) if isinstance(data, dict) else data
        if not notifications:
            print("[DEBUG] No 'notifications' key found or empty")
            return
        print(f"[DEBUG] Processing {len(notifications)} notifications in trending ray")
        chat_ids = get_enabled_chats('trending_ray')
        for notification in notifications:
            nid = notification.get('notificationId')
            if not nid:
                continue
            print(f"[DEBUG] Checking notification {nid}")
            c.execute("SELECT * FROM posted_notifications WHERE notification_id=?", (nid,))
            if not c.fetchone():
                message, address, image_url = format_trending(notification, "TRENDING RAY")
                if message is None:
                    continue
                print(f"[DEBUG] Token address for notification {nid}: {address}")
                # Double check: skip if token is already posted
                c.execute("SELECT * FROM posted_tokens WHERE token_address=?", (address,))
                if c.fetchone():
                    print(f"[DEBUG] Token {address} already posted, skipping notification {nid}")
                    continue
                keyboard = get_token_keyboard(address)
                for chat_id in chat_ids:
                    thread_id = get_thread_id(chat_id, 'trending_ray')
                    print(f"[DEBUG] Sending message for notification {nid} (token {address}) to chat {chat_id}, thread {thread_id}")
                    try:
                        if image_url and is_valid_image_url(image_url):
                            await bot.send_photo(
                                chat_id=chat_id,
                                message_thread_id=thread_id,
                                photo=image_url,
                                caption=message,
                                parse_mode='Markdown',
                                reply_markup=keyboard
                            )
                        else:
                            await bot.send_message(
                                chat_id=chat_id,
                                message_thread_id=thread_id,
                                text=message,
                                parse_mode='Markdown',
                                reply_markup=keyboard,
                                disable_web_page_preview=True
                            )
                        print(f"[DEBUG] Successfully sent message for notification {nid} (token {address})")
                        c.execute("INSERT INTO posted_notifications VALUES (?)", (nid,))
                        c.execute("INSERT OR IGNORE INTO posted_tokens VALUES (?)", (address,))
                        conn.commit()
                        print(f"[DEBUG] Inserted notification {nid} and token {address} into database")
                    except RetryAfter as e:
                        print(f"[DEBUG] Flood control exceeded for notification {nid} (token {address}). Retrying in {e.retry_after} seconds.")
                        time.sleep(e.retry_after)
                        if image_url and is_valid_image_url(image_url):
                            await bot.send_photo(
                                chat_id=chat_id,
                                message_thread_id=thread_id,
                                photo=image_url,
                                caption=message,
                                parse_mode='Markdown',
                                reply_markup=keyboard
                            )
                        else:
                            await bot.send_message(
                                chat_id=chat_id,
                                message_thread_id=thread_id,
                                text=message,
                                parse_mode='Markdown',
                                reply_markup=keyboard,
                                disable_web_page_preview=True
                            )
                        print(f"[DEBUG] Successfully sent message for notification {nid} (token {address}) after retry")
                        c.execute("INSERT INTO posted_notifications VALUES (?)", (nid,))
                        c.execute("INSERT OR IGNORE INTO posted_tokens VALUES (?)", (address,))
                        conn.commit()
                        print(f"[DEBUG] Inserted notification {nid} and token {address} into database after retry")
                    except Exception as e:
                        print(f"[DEBUG] Error sending message for notification {nid} (token {address}) to chat {chat_id}, thread {thread_id}: {e}")
                time.sleep(1)
            else:
                print(f"[DEBUG] Notification {nid} already posted, skipping.")

async def check_graduated(bot):
    print("[DEBUG] Starting check_graduated")
    data = get_meme_zone_filtered()
    if data:
        print(f"[DEBUG] Processing {len(data)} graduated tokens")
        chat_ids = get_enabled_chats('graduated')
        for chat_id in chat_ids:
            for token in data[:5]:
                address = token['address']
                print(f"[DEBUG] Checking token {address} in graduated")
                c.execute("SELECT * FROM posted_tokens WHERE token_address=?", (address,))
                if not c.fetchone():
                    print(f"[DEBUG] Token {address} not posted yet, preparing to send")
                    message, address, image_url = format_graduated_or_trending(token)
                    keyboard = get_token_keyboard(address)
                    thread_id = get_thread_id(chat_id, 'graduated')
                    print(f"[DEBUG] Sending message for token {address} to chat {chat_id}, thread {thread_id}")
                    try:
                        if image_url and is_valid_image_url(image_url):
                            await bot.send_photo(
                                chat_id=chat_id,
                                message_thread_id=thread_id,
                                photo=image_url,
                                caption=message,
                                parse_mode='Markdown',
                                reply_markup=keyboard
                            )
                        else:
                            await bot.send_message(
                                chat_id=chat_id,
                                message_thread_id=thread_id,
                                text=message,
                                parse_mode='Markdown',
                                reply_markup=keyboard,
                                disable_web_page_preview=True
                            )
                        print(f"[DEBUG] Successfully sent message for token {address}")
                        c.execute("INSERT INTO posted_tokens VALUES (?)", (address,))
                        conn.commit()
                        print(f"[DEBUG] Inserted token {address} into posted_tokens")
                    except RetryAfter as e:
                        print(f"[DEBUG] Flood control exceeded for token {address}. Retrying in {e.retry_after} seconds.")
                        time.sleep(e.retry_after)
                        if image_url and is_valid_image_url(image_url):
                            await bot.send_photo(
                                chat_id=chat_id,
                                message_thread_id=thread_id,
                                photo=image_url,
                                caption=message,
                                parse_mode='Markdown',
                                reply_markup=keyboard
                            )
                        else:
                            await bot.send_message(
                                chat_id=chat_id,
                                message_thread_id=thread_id,
                                text=message,
                                parse_mode='Markdown',
                                reply_markup=keyboard,
                                disable_web_page_preview=True
                            )
                        print(f"[DEBUG] Successfully sent message for token {address} after retry")
                        c.execute("INSERT INTO posted_tokens VALUES (?)", (address,))
                        conn.commit()
                        print(f"[DEBUG] Inserted token {address} into posted_tokens after retry")
                    except Exception as e:
                        print(f"[DEBUG] Error sending message for token {address} to chat {chat_id}, thread {thread_id}: {e}")
                else:
                    print(f"[DEBUG] Token {address} already posted, skipping.")
                time.sleep(1)

async def check_about_to_graduate(bot):
    print("[DEBUG] Starting check_about_to_graduate")
    data = get_about_to_graduate()
    if data:
        print(f"[DEBUG] Processing {len(data)} about-to-graduate tokens")
        chat_ids = get_enabled_chats('bonding')
        for token in data:
            address = token['address']
            print(f"[DEBUG] Checking token {address} in about-to-graduate")
            c.execute("SELECT * FROM posted_tokens WHERE token_address=?", (address,))
            if not c.fetchone():
                print(f"[DEBUG] Token {address} not posted yet, preparing to send")
                message, address, image_url = format_about_to_graduate(token)
                keyboard = get_token_keyboard(address)
                for chat_id in chat_ids:
                    thread_id = get_thread_id(chat_id, 'bonding')
                    print(f"[DEBUG] Sending message for token {address} to chat {chat_id}, thread {thread_id}")
                    try:
                        if image_url and is_valid_image_url(image_url):
                            await bot.send_photo(
                                chat_id=chat_id,
                                message_thread_id=thread_id,
                                photo=image_url,
                                caption=message,
                                parse_mode='Markdown',
                                reply_markup=keyboard
                            )
                        else:
                            await bot.send_message(
                                chat_id=chat_id,
                                message_thread_id=thread_id,
                                text=message,
                                parse_mode='Markdown',
                                reply_markup=keyboard,
                                disable_web_page_preview=True
                            )
                        print(f"[DEBUG] Successfully sent message for token {address}")
                        c.execute("INSERT INTO posted_tokens VALUES (?)", (address,))
                        conn.commit()
                        print(f"[DEBUG] Inserted token {address} into posted_tokens")
                    except RetryAfter as e:
                        print(f"[DEBUG] Flood control exceeded for token {address}. Retrying in {e.retry_after} seconds.")
                        time.sleep(e.retry_after)
                        if image_url and is_valid_image_url(image_url):
                            await bot.send_photo(
                                chat_id=chat_id,
                                message_thread_id=thread_id,
                                photo=image_url,
                                caption=message,
                                parse_mode='Markdown',
                                reply_markup=keyboard
                            )
                        else:
                            await bot.send_message(
                                chat_id=chat_id,
                                message_thread_id=thread_id,
                                text=message,
                                parse_mode='Markdown',
                                reply_markup=keyboard,
                                disable_web_page_preview=True
                            )
                        print(f"[DEBUG] Successfully sent message for token {address} after retry")
                        c.execute("INSERT INTO posted_tokens VALUES (?)", (address,))
                        conn.commit()
                        print(f"[DEBUG] Inserted token {address} into posted_tokens after retry")
                    except Exception as e:
                        print(f"[DEBUG] Error sending message for token {address} to chat {chat_id}, thread {thread_id}: {e}")
                time.sleep(1)
            else:
                print(f"[DEBUG] Token {address} already posted, skipping.")

# Command Handlers (Updated /start Message)
async def start(update: Update, context: CallbackContext):
    chat_id = str(update.message.chat.id)
    enable_all_modules(chat_id)
    message = (
        "All alerts enabled. To disable a specific module, use /stop [module].\n"
        "Available modules: meme, trending_pump, trending_ray, graduated, bonding\n"
        "Example: /stop meme\n\n"
        "Join our main group @vaultboyportal for more updates!"
    )
    await update.message.reply_text(message)

# Scheduler Functions
async def schedule_main_checks(context: CallbackContext):
    print("[DEBUG] Running schedule_main_checks")
    try:
        bot = context.bot
        await check_meme_zone(bot)
        await check_trending_pump(bot)
        await check_trending_ray(bot)
        await check_graduated(bot)
        await check_about_to_graduate(bot)
    except Exception as e:
        print(f"[DEBUG] Error in schedule_main_checks: {e}")

# Main Function
def main():
    print("[DEBUG] Initializing bot")
    application = Application.builder().token(BOT_TOKEN).build()

    application.add_handler(CommandHandler("start", start))

    job_queue = application.job_queue
    print("[DEBUG] Setting up job queue")
    job_queue.run_repeating(schedule_main_checks, interval=60, first=0)

    print("[DEBUG] Starting bot polling")
    application.run_polling()

if __name__ == "__main__":
    main()
